baseUrl = "https://web.alistetechnologies.com"
baseUrl2 = "https://a3.alistetechnologies.com"
wssUrl = "https://a2.alistetechnologies.com/?"
loginUrl = "https://web.alistetechnologies.com/v2/auth/login"
homeDetailsUrl = "https://a3.alistetechnologies.com/api/fetch/house2"
commandUrl = "https://a3.alistetechnologies.com/v3/device/control"
region = "ap-south-1"
cognitoUrl = f"https://cognito-identity.{region}.amazonaws.com"
identityId = f"{region}:4b454fb8-df98-4b94-a5ba-937169c43ad0"
iotId = "a1pv71f2h6guqz-ats"
